<?php
/**
 * $Horde: forwards/lib/Driver/customsql.php,v 1.3 2005/01/03 14:34:49 jan Exp $
 *
 * Copyright 2004-2005 Kendrick Vargas <ken@hudat.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * Forwards_Driver_sql:: implements the Forwards_Driver API for SQL servers.
 *
 * @author  Kendrick Vargas <ken@hudat.com>
 * @version $Revision: 1.3 $
 * @since   Forwards 3.0
 * @package Forwards
 */
class Forwards_Driver_customsql extends Forwards_Driver {

    /** file pointer to the sql connection. */
    var $_db;

    /** boolean which contains state of sql connection */
    var $_connected = false;

    /**
     * Constructs a new sql Forwards_Driver object.
     *
     * @param array  $params        A hash containing connection parameters.
     */
    function Forwards_Driver_customsql($params = null)
    {
        if (is_null($params)) {
            $params = Horde::getDriverConfig('', 'sql');
        }
        parent::Forwards_Driver($params);
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean true on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            return $this->_db->disconnect();
        }

        return true;
    }

    /**
     * Begins forwarding of mail for a user.
     *
     * @param string        $user     The username to enable forwarding for.
     * @param string        $realm    The realm of the user.
     * @param string        $pass     The password of the user.
     *
     * @param string        $target   The email address that mail should
     *                                be forwarded to.
     *
     * @param optional boolean $keeplocal A flag that if true causes a
     *                                    copy of forwarded email to be
     *                                     kept in the local mailbox.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function enableForwarding($user, $realm, $password, $target, $keeplocal)
    {
        // _connect() will die with Horde::fatal() upon failure.
        $this->_connect();
        $keeplocal = $keeplocal ? "Yes" : "No";

        // Build the SQL query.
        $query = $this->_params['query_set'];
        $query = str_replace("\U", $this->_db->quote(Auth::getAuth()), $query);
        $query = str_replace("\T", $this->_db->quote($target), $query);
        $query = str_replace("\L", $this->_db->quote($keeplocal), $query);
        $query = str_replace("\P", $this->_db->quote($password), $query);

        // Execute the query.
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            if ($result === DB_OK) {
                $this->_disconnect();
                return true;
            }
        }

        $this->_disconnect();
        return false;
    }

    /**
     * Stops forwarding of mail for a user.
     *
     * @param string        $user      The username of the user.
     * @param string        $realm    The realm of the user.
     * @param string        $pass      The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function disableForwarding($user, $realm, $password)
    {
        // _connect() will die with Horde::fatal() upon failure.
        $this->_connect();

        // Build the SQL query.
        $query = $this->_params['query_disable'];
        $query = str_replace("\U", $this->_db->quote(Auth::getAuth()), $query);
        $query = str_replace("\P", $this->_db->quote($password), $query);

        // Execute the query.
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            if ($result === DB_OK) {
                $this->_disconnect();
                return true;
            }
        }

        $this->_disconnect();
        return false;
    }

    /**
     * Retrieves status of mail redirection for a user
     *
     * @param string        $user      The username of the user to check.
     *
     * @param string        $realm    The realm of the user to check.
     *
     * @return mixed        Returns 'Y' if forwarding is enabled, or false.
     */
    function isEnabledForwarding($user, $realm, $password)
    {
        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // get current details
        $current_details = $this->_getUserDetails($myuser, $realm, $password);
        if ($current_details === false) {
            return false;
        } else {
            return 'Y';
        }

    }

    /**
     * Returns true if a local copy of forwarded messages is being kept
     *
     * @param string        $user      The username of the user to check.
     *
     * @param string        $realm    The realm of the user to check.
     *
     * @return boolean      Returns true if retain local copy is enabled
     *                      else false.
     */
    function isKeepLocal($user, $realm, $password)
    {
        // Build username.
        $myuser = $this->_buildUsername($user, $realm);

        // Get current details.
        $current_details = $this->_getUserDetails($myuser, $realm, $password);
        if ($current_details === false) {
            return false;
        }

        // Check retain copy flag.
        if ($current_details[$this->_params['column_keeplocal']] == 'Y') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Retrieves current target of mail redirection
     *
     * @param string        $user      The username of the user.
     * @param string        $realm    The realm of the user.
     *
     * @return string   A string of current forwarding address or false.
     */
    function currentTarget($user, $realm, $password)
    {
        $current_details = $this->_getUserDetails($user, $realm, $password);

        // Check current forwarding mail address.
        $target = $current_details[$this->_params['column_target']];
        return $target;
    }

    /**
     * Retreive relevant line from sql server
     *
     * @param   $user            The username for which to retrieve details.
     * @param   $realm           The realm (domain) for the user.
     * @param   $password        The password for user.
     *
     * @return  Mixed            Mysql result resource or (boolean) False.
     */
    function _getUserDetails($user, $realm, $password)
    {
        // _connect() will die with Horde::fatal() upon failure.
        $this->_connect();

        // Build the SQL query.
        $query = $this->_params['query_select'];
        $query = str_replace("\U", $this->_db->quote(Auth::getAuth()), $query);
        $query = str_replace("\P", $this->_db->quote($password), $query);

        // Execute the query.
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);

            $this->_disconnect();
            if (is_array($row)) {
                return $row;
            } else {
                $result->free();
                return false;
            }
        }

        $this->_disconnect();
        return false;
    }

    /**
     * Builds a username based on presence of realm.
     *
     * @return string  $user@$realm or $user.
     */
    function _buildUsername($user, $realm)
    {
        if ($realm === 'default' ||
            $realm === '') {
            return $user;
        } else {
            return $user . '@' . $realm;
        }
    }

    /**
     * Do an sql connect and login as user with privilege to change
     * passwd.
     *
     * @return   boolean   True or False based on success of connect
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'server',
                array('phptype', 'hostspec', 'username', 'database', 'query_select', 'query_set', 'query_disable', 'column_target', 'column_keeplocal'),
                'authentication SQL');

            // Connect to SQL server using supplied parameters.
            include_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                      array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal(PEAR::raiseError(_("Unable to connect to SQL server.")), __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

}
