<?php

require_once 'SOAP/Client.php';

/**
 * $Horde: forwards/lib/Driver/soap.php,v 1.3 2005/01/03 14:34:49 jan Exp $
 *
 * Copyright 2004-2005 Marc Jauvin <marc@r4l.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * Forwards_Driver_soap:: implements the Forwards_Driver API for SOAP
 * services.
 *
 * @author  Marc Jauvin <marc@r4l.com>
 * @since   Forwards 3.0
 * @package Forwards
 */
class Forwards_Driver_soap extends Forwards_Driver {

    /**
     * The information that was retrieved from the SOAP server.
     *
     * @var array $_current_details
     */
    var $_current_details;

    /**
     * Is the SOAP client initialized?
     *
     * @var boolean $_initialized
     */
    var $_initialised = false;

    /**
     * Constructs a new Forwards_Driver_soap object.
     *
     * @param array $params  A hash containing connection parameters.
     */
    function Forwards_Driver_soap($params = null)
    {
        if (is_null($params)) {
            $params = Horde::getDriverConfig('', 'soap');
        }
        parent::Forwards_Driver($params);
    }

    /**
     * Initialise the SOAP connection object.
     *
     * @return boolean  True or False based on success of connect.
     */
    function _initialise()
    {
        if (!$this->_initialised) {
            Horde::assertDriverConfig($this->_params, 'server',
                array(
                    'soap_proxy',
                    'soap_uri',
                    'fwd_fetch_func',
                    'fwd_enable_func',
                    'fwd_disable_func',
                    'username_varname',
                    'target_varname',
                    'keeplocal_varname',
                ),
                'SOAP initialisation'
            );

            $this->_client = &new SOAP_Client($this->_params['soap_proxy']);
            $this->_client->setOpt('curl', CURLOPT_FAILONERROR, false);
            $this->_client->setOpt('curl', CURLOPT_SSL_VERIFYPEER, false);

            $this->_initialised = true;
        }

        return true;
    }

    /**
     * Retrieve relevant line from SOAP server.
     *
     * @param $user   The username for which to retrieve details.
     * @param $realm  The realm (domain) for the user.
     *
     * @return mixed  SOAP result resource or (boolean) False.
     */
    function _getUserDetails($user, $realm, $password)
    {
        // _initialise() will die with Horde::fatal() upon failure.
        $this->_initialise();

        if ($this->_current_details) {
            return $this->_current_details;
        }

        // Build username.
        $account = $this->_buildUsername($user, $realm);

        $data[$this->_params['username_varname']] = $account;
        $args = &new SOAP_Value("args", "array", $data);

        $result = $this->_client->call($this->_params['fwd_fetch_func'], $args, $this->_params['soap_uri']);

        if (is_a($result, 'PEAR_Error')) {
            $this->_current_details = array();
            return false;
        } else {
            $this->_current_details = get_object_vars($result);
            return $this->_current_details;
        }
    }

    /**
     * Retrieves status of mail redirection for a user
     *
     * @param string        $user     The username of the user to check.
     *
     * @param string        $realm    The realm of the user to check.
     *
     * @return boolean      Returns true if forwarding is enabled for the
     *                      user or false if forwarding is currently
     *                      disabled for the user.
     */
    function isEnabledForwarding($user, $realm, $password)
    {
        // Get current details.
        $current_details = $this->_getUserDetails($user, $realm, $password);
        if ($current_details === false) {
            return false;
        }

        // Check forwarding flag.
        if ($current_details[$this->_params['target_varname']]) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Returns true if a local copy of forwarded messages is being kept
     *
     * @param string        $user      The username of the user to check.
     *
     * @param string        $realm    The realm of the user to check.
     *
     * @return boolean      Returns true if retain local copy is enabled
     *                      else false.
     */
    function isKeepLocal($user, $realm, $password)
    {
        // Get current details.
        $current_details = $this->_getUserDetails($user, $realm, $password);
        if ($current_details === false) {
            return false;
        }

        if ($current_details[$this->_params['keeplocal_varname']]) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Retrieves current target of mail redirection
     *
     * @param string        $user      The username of the user.
     * @param string        $realm    The realm of the user.
     *
     * @return string   A string of current forwarding address or false.
     */
    function currentTarget($user, $realm, $password)
    {
        $current_details = $this->_getUserDetails($user, $realm, $password);

        if ($current_details[$this->_params['target_varname']]) {
            return $current_details[$this->_params['target_varname']];
        } else {
            return false;
        }
    }

    /**
     * Begins forwarding of mail for a user.
     *
     * @param string        $user     The username to enable forwarding for.
     * @param string        $realm    The realm of the user.
     * @param string        $pass     The password of the user.
     *
     * @param string        $target   The email address that mail should
     *                                be forwarded to.
     *
     * @param optional boolean $keeplocal A flag that if true causes a
     *                                    copy of forwarded email to be
     *                                     kept in the local mailbox.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function enableForwarding($user, $realm, $password, $target, $keeplocal)
    {
        // _initialise() will die with Horde::fatal() upon failure.
        $this->_initialise();

        // Build username.
        $account = $this->_buildUsername($user, $realm);

        $data[$this->_params['username_varname']] = $account;
        if ($password && $this->_params['password_varname']) {
            $data[$this->_params['password_varname']] = $password;
        }
        $data[$this->_params['target_varname']] = $target;
        $data[$this->_params['keeplocal_varname']] = ($keeplocal === 'on') ? 1 : 0;

        $args = &new SOAP_Value('args', 'array', $data);

        $result = $this->_client->call($this->_params['fwd_enable_func'], $args, $this->_params['soap_uri']);
        if (is_a($result, 'PEAR_Error')) {
            $this->_current_details = array();
            return false;
        }

        return true;
    }

    /**
     * Stops forwarding of mail for a user.
     *
     * @param string $user   The username of the user.
     * @param string $realm  The realm of the user.
     * @param string $pass   The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    function disableForwarding($user, $realm, $password)
    {
        // _initialise() will die with Horde::fatal() upon failure.
        $this->_initialise();

        // Build username.
        $account = $this->_buildUsername($user, $realm);

        $data[$this->_params['username_varname']] = $account;
        if ($password && $this->_params['password_varname']) {
            $data[$this->_params['password_varname']] = $password;
        }

        $args = &new SOAP_Value("args", "array", $data);

        $result = $this->_client->call($this->_params['fwd_disable_func'], $args, $this->_params['soap_uri']);
        if (is_a($result, 'PEAR_Error')) {
            $this->_current_details = array();
            return false;
        }

        return true;
    }

    /**
     * Builds a username based on presence of realm.
     *
     * @return string  $user@$realm or $user.
     */
    function _buildUsername($user, $realm)
    {
        if ($realm === 'default' ||
            $realm == '') {
            return $user;
        } else {
            return $user . '@' . $realm;
        }
    }

}
